@@ -340,14 +340,9 @@ void pqServerConfiguration::parseSshPortForwardingXML()
             this->PortForwardingLocalPort = QString(sshForwardXML->GetAttributeOrDefault(
               "local", QString::number(this->resource().port()).toUtf8().data()));
 
-            // rc connection do not need to set the actual URI
-            // as connection is managed server side with pvserver arguments
-            if (!resource.isReverse())
-            {
-              resource.setHost("localhost");
-              resource.setPort(this->PortForwardingLocalPort.toInt());
-              this->ActualURI = resource.schemeHostsPorts().toURI();
-            }
+            resource.setHost("localhost");
+            resource.setPort(this->PortForwardingLocalPort.toInt());
+            this->ActualURI = resource.schemeHostsPorts().toURI();
           }
         }
       }
@@ -504,13 +499,18 @@ QString pqServerConfiguration::sshFullCommand(
   vtkPVXMLElement* sshForwardXML = sshConfigXML->FindNestedElementByName("PortForwarding");
   if (sshForwardXML)
   {
-    QString sshFlag = "-L ";
     if (this->resource().isReverse())
     {
-      sshFlag = "-R ";
+      // Reverse tunnelling
+      sshStream << "-R " << QString::number(this->resource().port())
+                << ":localhost:" << this->PortForwardingLocalPort << " ";
+    }
+    else
+    {
+      // Forward tunnelling
+      sshStream << "-L " << this->PortForwardingLocalPort
+                << ":localhost:" << QString::number(this->resource().port()) << " ";
     }
-    sshStream << sshFlag << this->PortForwardingLocalPort
-              << ":localhost:" << QString::number(this->resource().port()) << " ";
   }
 
   QString sshUser = sshConfigXML->GetAttributeOrDefault("user", "");
@@ -518,7 +518,7 @@ QString pqServerConfiguration::sshFullCommand(
   {
     sshStream << sshUser << "@";
   }
-  sshStream << this->resource().host();
+  sshStream << this->resource().host() << " ";
   return sshFullCommand;
 }
 
