@@ -232,6 +232,13 @@ void vtkOpenXRRenderWindowInteractor::PollXrActions()
     }
   }
 
+  auto renWin = vtkOpenXRRenderWindow::SafeDownCast(this->RenderWindow);
+  if (!renWin)
+  {
+    vtkErrorMacro("Unable to retrieve the OpenXR render window !");
+    return;
+  }
+
   // Construct the event data that contains position and orientation of each hand
   double pos[3] = { 0.0 };
   double ppos[3] = { 0.0 };
@@ -261,6 +268,13 @@ void vtkOpenXRRenderWindowInteractor::PollXrActions()
       this->SetPhysicalEventPosition(ppos[0], ppos[1], ppos[2], pointerIndex);
       this->SetWorldEventPosition(pos[0], pos[1], pos[2], pointerIndex);
       this->SetWorldEventOrientation(wxyz[0], wxyz[1], wxyz[2], wxyz[3], pointerIndex);
+
+      // Update DeviceToPhysical matrices, this is a read-write access!
+      vtkMatrix4x4* devicePose = renWin->GetDeviceToPhysicalMatrixForDevice(edHand->GetDevice());
+      if (devicePose)
+      {
+        vtkOpenXRUtilities::SetMatrixFromXrPose(devicePose, *handPose);
+      }
     }
   }
 
@@ -292,13 +306,6 @@ void vtkOpenXRRenderWindowInteractor::PollXrActions()
   // vtkOpenXRManager to retrieve the "real" head pose (for now we use the left
   // eye direction retrieved in vtkOpenXRRenderWindow::UpdateHMDMatrixPose,
   // that is close).
-  auto renWin = vtkOpenXRRenderWindow::SafeDownCast(this->RenderWindow);
-  if (!renWin)
-  {
-    vtkErrorMacro("Unable to retrieve the OpenXR render window !");
-    return;
-  }
-
   // Retrieve headset pose matrix in physical coordinates and convert to position and orientation
   // in world coordinates
   vtkMatrix4x4* poseMatrix =
@@ -456,6 +463,13 @@ void vtkOpenXRRenderWindowInteractor::HandleVector2fAction(
   }
 }
 
+//------------------------------------------------------------------------------
+void vtkOpenXRRenderWindowInteractor::AddAction(
+  const std::string& path, const vtkCommand::EventIds& eid, bool vtkNotUsed(isAnalog))
+{
+  this->AddAction(path, eid);
+}
+
 //------------------------------------------------------------------------------
 void vtkOpenXRRenderWindowInteractor::AddAction(
   const std::string& path, const vtkCommand::EventIds& eid)
@@ -471,6 +485,13 @@ void vtkOpenXRRenderWindowInteractor::AddAction(
   am->UseFunction = false;
 }
 
+//------------------------------------------------------------------------------
+void vtkOpenXRRenderWindowInteractor::AddAction(const std::string& path, bool vtkNotUsed(isAnalog),
+  const std::function<void(vtkEventData*)>& func)
+{
+  this->AddAction(path, func);
+}
+
 //------------------------------------------------------------------------------
 void vtkOpenXRRenderWindowInteractor::AddAction(
   const std::string& path, const std::function<void(vtkEventData*)>& func)
@@ -625,6 +646,9 @@ bool vtkOpenXRRenderWindowInteractor::LoadActions(const std::string& actionFilen
                   << ", localizedName=" << localizedName);
 
     XrActionType xrActionType = this->GetActionTypeFromString(type);
+    // `GetActionTypeFromString` casts `0` when it fails, so the variant in
+    // this case is not "valid", but `clang-tidy` doesn't know that.
+    // NOLINTNEXTLINE(bugprone-non-zero-enum-to-bool-conversion)
     if (!xrActionType)
     {
       return false;
